//+------------------------------------------------------------------+
//|                                                        BcLib.mq4 |
//|                                                 Bogdan Caramalac |
//|                                     mailto:fxeconomist@yahoo.com |
//+------------------------------------------------------------------+

#property show_inputs

#include <stderror.mqh>

//Completion to stderror.mqh
#define ERR_TRADE_EXPIRATION_DENIED	147	//Expirations are denied by broker.

//Constants
#define pi      3.1415926535
#define base_e  2.718281828

//Constants used for ComputeWidth() function
#define BEST_REGRESSION           0
#define LINEAR_REGRESSION         1
#define EXPONNENTIAL_REGRESSION2  2      


//system variables
string WordsSeparatorsList=" ";
int LinesDrawn=0; 
string SplashText="";
int LinesOfText=0;
int TextDisplayWindow=0;
string TextFontName="Arial"; 
int TextFontSize=10;
int TextFontSpacing=4;
int TextColor=White;
bool TextUnder=false;
int _DefaultTextStartX=3;
int _DefaultTextStartY=10;
int TextStartX=3;
int TextStartY=13;
int TextNowY=13;

 //***************************************************************
 // I N T E R F A C E
 //***************************************************************
 /*
 
 string StringElement(string s,int pos)           //the <pos> element from s
 int    At(string tosearch,string into,int pos)   //the position of the <pos>th occurence of tosearch inside into
 int    CharInList(string element,string list)    //True if element is in list
 string RevStr(string stri)                       //the reverse of a given string
 string SubstrBetween(string stri,int p1,int p2)  //the substring from stri, starting from p1, ending at p2
 string Ltrim(string stri)                        //removes separators (see WordsSeparatorsList) from the left side of a string
 string Rtrim(string stri)                        //removes separators (see WordsSeparatorsList) from the right side of a string
 string Alltrim(string stri)                      //removes separators (see WordsSeparatorsList) from the both sides of a string
 string AlltrimSpacesOnly(string s)               //same as before, but only spaces and tabs (like MetaTrader funcs)
 int    Occurs(string tosearch,string into)       //the number of tosearch occurences in into
 string StrTran(string sir,string searchfor,      //transforms sir,replacing howmany occurences of searchfor with replacewith starting from
         string replacewith,int from,int howmany)                                  
 string Replicate(string s,int n)                 //replicates given string for n times
 string Upper(string s)                           //converts string to uppercase
 string Lower(string s)                           //converts string to lowercase
 string CRLF()                                    //carriage return & line feed

 //   
 // Phrase functions ; for these, words count from 1..n, not from 0..n-1
 // All these functions take in consideration WordsSeparatorsList
 //

 int    CntStrWords(string stri)                             //counts the number of words in the string 
 string ReturnNthWord(string stri,int n)                     //returns the nth word from given string
 string CopyPhrase(string phra,int firstword,int howmany)    //rebuilds phrase with howmany words starting from firstword
 string SubPhrase(string phra,int firstword,int lastword)    //rebuilds phrase with words between firstword and lastword

 //***************************************************************
 // Splash functions
 //***************************************************************    

 void ClearSplash()
 void Splash(string s)
 void SplashLn(string s)
 int SplashScreen(int buttons=EMPTY)
 
 //***************************************************************
 // Text display functions
 //***************************************************************    

 void ClearText()                                        //clears all text objects
 void RefreshText()                                      //refreshes text to reflect TextUnder changes
 void SetDisplayWindow(int handle)                       //sets the display window to use
 void SetTextUnder(bool under);                          //as true or false, sets text under or over graph
 void SetTextStart(int startx, int starty);              //sets text start Point
 void SetTextParameters(string font,int size);           //sets text parameters      
 void SetTextSpacing(int spacing)                        //sets spacing between rows
 void SetTextColor(int colour)                           //sets text color      
 void Write(string s)                                    //write text at current position, next text will continue
 void WriteLn(string s)                                  //writes text at current position, next text on next line
 void Say(int row,string s)                              //writes text at given row
 void SayAppend(int row,string s)                        //appends text at given row
 

 //***************************************************************
 // Time & date functions
 //***************************************************************    
    
 int    PackTime(datetime d,int yearbase)                    //packs given date into an int, including Seconds
                                                             //yearbase is subtracted from year, since int type is too small

 //***************************************************************
 // Math functions 
 //***************************************************************

 int    Round(double i)                                            //Converts double to integer
 double RoundTo(double i,int digits)                               //Rounds integer to specified number of digits
 int    Truncate(double a)                                         //truncates the double to integer as trunc(3.7)=3 , truc(-3.7)=3
 int    MathDiv(double a,double b)                                  
 bool   Between(double v,double a,double b)                        //True if v is between a and b
 int    Sgn(double x)                                              //the sign of x, as -1, 0, or 1
 double DtoR(double x)                                             //converts degrees to radians
 double RtoD(double x)                                             //converts radians to degrees
 double ATan2(double x,double y)                                   //the angle of the given point with axis origin
 void   DetLine(double x1,double y1,double x2,                     //determines the line (y=M*x+N) going thru (x1,y1) and (x2,y2)
            double y2,double &m,double &n)                         
 void   DetIntersection(double m1,double n1,double m2,double n2,   //determines intersection of two lines
            double &x,double &y)                             
 void   TranslateLine(double m,double n,double x,double y,         //translates given line, (keeping slope) to go thru (x,y)
            double &m2,double &n2)
 double Distance(double x1,double y1,double x2,double y2)          //the distance between two points
 void   RotatePoint(double xp,double yp,double xc,double yc,       //given the (xc,yc) origin , rotates (x,y) by adding AngleDisplacement degrees
            double AngleDisplacement,double &newx,double &newy)             

 //***************************************************************
 // Statistics functions 
 //***************************************************************     
 
 void   StatParameters(double array[],int p1,int p2,               //calculates the average and variance for values between p1 and p2 in array
          double &avg,double &variance,double &stddev)    
 void   LinReg(double x[],double y[],int p1,int p2,double &a,      //may calculate the regression line of x and y series;
          double &b,double &variance,double &stddev,bool UseY)     //from p1 towards p2 on both arrays;
                                                                   //returns the a and b parameters of regression line, b being the angular coefficient
                                                                   //y=bx+a; if UseY=false, uses x as (x+y)/2, and y = p1...pn
 void   ExpReg2(double x[],double y[],int p1,int p2,double &a,     //a sort of exponnential regression; in fact a linear regression of logarithmed values                                                   
          double &b,double &variance,double &stddev,bool UseY)     //works like RegLin, but find out regression curve by calculating MathPow(base_e,b*x+a)
                                                                   //this time variance and stddev are meaningful only if UseY=false                                                                   
                                                                   
                                                                 

 //***************************************************************
 // T.A. stuff
 //***************************************************************

 string Line(int time0,double price0,int time1,double price1,
          int colour)                                              //Draws a line on current chart and returns object name if ok
 double ComputeWidth(int RegMethod,double Highs[],double Lows[],)  //calculates market width between high and low arrays; may be prices, moving avgs, etc.
          int p1,int p2,double &m,double &n, double &variance,
          double &stddev, int &MethodChosen)
          
//***************************************************************
 // Other functions
 //***************************************************************                    
 
 bool ContractExists(string symbol)                                //returns True if a contract exists
 double ConvertCurrency(double pips,string contract,
    string currency,double forceprice,string &errormsg)            //converts pips gained on one contract to other currency, at market rate, if forceprice=0 
 
 //***************************************************************
 // Translation functions
 //*************************************************************** 

 string Bool2Str(int boo)                                          //translates Bool to String
 string TimeFrame2Str(int timef)                                   //translates TimeFrame to String
 int    Str2TimeFrame(string timef)                                //translates String 2 TimeFrame
 string OrderType2Str(int otype)                                   //translates order type 2 string
 void   TranslateError(int errcode,string &ErrType,                //translates errors retrieving type,text and explanation; only
           string &ErrText,string &ErrExplanation)                 //server errors are explained
  
*/

 //***************************************************************
 // I M P L E M E N T A T I O N
 //***************************************************************



 //***************************************************************
 // String functions 
 //***************************************************************
 
string StringElement(string s,int pos)
   {
   int g=0;
   string empty="";
   string given="x";
   if (pos<0||pos>StringLen(s)-1)
      return(empty);
   else
      {
      g=StringGetChar(s,pos);
      given=StringSetChar(given,0,g);
      return(given);   
      }
   }
      
   
 

int At(string tosearch,string into,int pos)
   {
   int res=-1;
   int posi=0;
   int l=StringLen(tosearch);
   
   for (int k=0;k<=StringLen(into)-l;k++)
       {          
       if (StringSubstr(into,k,l)==tosearch)
          {
          posi=posi+1;
          if (posi==pos)
             {
             res=k;
             break;
             }
          k=k+l-1;
          }//if (StringSubstr(into,k,l)==tosearch)
       }
   return(res);
   } 
 
int CharInList(string element,string list)
   {
   int res=-1;
   res=At(StringElement(element,0),list,1);
   return(res);
   }


string RevStr(string stri)
   {
   string resul="";
   int k;
   if (StringLen(stri)!=0)
      {
      for (k=StringLen(stri)-1;k>=0;k--)
          {          
          resul=StringConcatenate(resul,StringElement(stri,k));
          }
      }
   return(resul);
   }
   
string SubstrBetween(string stri,int p1,int p2)
   {
   string res="";
   if (p2<p1||p1>StringLen(stri))
      return(res);
   if (p2>StringLen(stri))
      p2=StringLen(stri);
   for (int k=p1;k<=p2;k++)
      {
      res=StringConcatenate(res,StringElement(stri,k));
      }
   return(res);
   }


      
string Ltrim(string stri)
   {
   string res="";
   int k=0;
   int final=0;
   if (StringLen(stri)==0)
      return(res);
   for (k=0;k<StringLen(stri);k++)
      {      
      if (CharInList(StringElement(stri,k),WordsSeparatorsList)==-1)//current char is not separator
         {
         final=k;//final result is cut here
         break;
         }      
      }
   if (final<StringLen(stri))
      res=SubstrBetween(stri,final,StringLen(stri));
   return(res);
   }

    
      
string Rtrim(string stri)
   {
   string res="";
   res=RevStr(Ltrim(RevStr(stri)));
   return(res);
   }
   
string Alltrim(string stri)
   {
   string res="";
   res=Rtrim(Ltrim(stri));
   return(res);
   }


string AlltrimSpacesOnly(string s)//normal Metatrader functions behaviour
   {
   string r;
   r=StringTrimLeft(StringTrimRight(s));
   return(r);
   }

int Occurs(string tosearch,string into)
   {
   int res=0;
   int l=StringLen(tosearch);
   int lito=StringLen(into);  
   for (int k=0;k<=lito-l;k++)
       {
       if (StringSubstr(into,k,l)==tosearch)
          {
          res=res+1;
          k=k+l-1;
          }
       if (k>lito-l)
          break;
       }
   return(res);
   }

string StrTran(string sir,string searchfor,string replacewith,int from,int howmany)
   {
   string res="";
   if (from<0)
      from=0;
   if (howmany==0)
      howmany=Occurs(searchfor,sir)-from+1;
   if (howmany<=0)
      return(res);
   int lsir=StringLen(sir);
   int lsf=StringLen(searchfor);
   int occnow=0;                
   int occend=from+howmany-1;
   string sirex="";
   for (int i=0;i<lsir;i++)
       {
       if (lsir-i<lsf)
          sirex=StringConcatenate(sirex,StringElement(sir,i));
       else
          {
          if (StringSubstr(sir,i,lsf)==searchfor)
             {
             occnow=occnow+1;
             if ((occnow>=from&&occnow<=occend))
                {
                sirex=StringConcatenate(sirex,replacewith);            
                i=i+lsf-1;
                }
             else                
                sirex=StringConcatenate(sirex,StringElement(sir,i));                
             }//endif if ((occnow>=from&&occnow<=occend))  
          else
             sirex=StringConcatenate(sirex,StringElement(sir,i));
          }//endif if (StringSubstr(sir,i,lsf)==searchfor)
      }//endfor
   return(sirex);   
   }   
   

string Replicate(string s,int n)
   {
   string res="";
   int k=1;
   if (n<1)
      return(res);                    
   for (k=1;k<=n;k++) 
       {
       res=StringConcatenate(res,s);
       }
    return(res);
    }          

string Upper(string s)
   {
   string res=s;
   int k=0;
   int ascii;
   int l=StringLen(res);
   if (l!=0)
      {
      for (int i=0;i<l;i++)
          {
          ascii=StringGetChar(res,i);
          if (ascii>=97&&ascii<=122)
             res=StringSetChar(res,i,ascii-32);
          }
      }
   return(res);
   }


string Lower(string s)
   {
   string res=s;
   int k=0;
   int ascii;
   int l=StringLen(res);
   if (l!=0)
      {
      for (int i=0;i<l;i++)
          {
          ascii=StringGetChar(res,i);          
          if (ascii>=65&&ascii<=90)
             res=StringSetChar(res,i,ascii+32);
          }
      }
   return(res);
   }

string CRLF()                                   
   {
   string res;
   res="  ";
   res=StringSetChar(res,1,13);
   res=StringSetChar(res,2,10);
   return(res);
   }

//   
// Phrase functions ; for these, words count from 1..n, not from 0..n-1
//

void FrontToken(string s,string &resu, string &rest)
   {
   int i,j,stl=0;
   string news,fwd="";   
   news=StringTrimLeft(s);
   fwd="";
   stl=StringLen(news);
   if (stl!=0)
   {
      for(i=0; i<stl; i++)
      {
          if (CharInList(StringElement(news,i),WordsSeparatorsList)!=-1)         
             break;
          else         
             fwd=StringConcatenate(fwd,StringSubstr(news,i,1));         
      }     
   }
   resu=fwd;
   if (StringLen(news)-StringLen(fwd)!=0)
      rest=StringSubstr(news,StringLen(fwd),StringLen(news)-StringLen(fwd));  
   else 
      {
      rest="";
      }
  }
    
    
       
int CntStrWords(string stri)
    {
    string stra=Alltrim(stri);
    int wrd=0;    
    string word="";
    string result="";
    while(true)
    {
        FrontToken(stra,word,result);
        if (StringLen(word)!=0)
           {
           wrd=wrd+1;
           stra=result;
           }
        else           
           break;           
    }
    return(wrd);
    }
        
           

string ReturnNthWord(string stri,int n)
    {
    string stra="";    
    int wrd=0;    
    int i=0;
    string word="";
    string wordset="";
    string result="";
    int nwrds=0;    
    stra=Alltrim(stri);
    nwrds=CntStrWords(stri);
    if (n>nwrds||n<1)
       {              
       return(word);
       }    
    while(True)
    {
        FrontToken(stra,word,result);
        if (StringLen(word)!=0) //didn't finish phrase
           {
           wrd=wrd+1;                       
           stra=result;
           if (wrd==n)
              {
              wordset=word;
              break;
              }
           }
        else           
           break;                           
    }
    return(wordset);
    }  
 
      
string CopyPhrase(string phra,int firstword,int howmany)
    {
    int k=0;
    string resul="";
    if (howmany==0)
       {
       howmany=CntStrWords(phra)-firstword+1;
       }
    if (firstword+howmany-1<firstword)
       return(resul);
    for (k=firstword;k<=firstword+howmany-1;k++)
       {
       resul=resul+ReturnNthWord(phra,k);       
       if (k!=firstword+howmany-1)
          resul=resul+StringElement(WordsSeparatorsList,0);
       }
    return(resul);
    }
    
    
string SubPhrase(string phra,int firstword,int lastword)
    {
    int k=0;
    string resul="";
    if (lastword==0)
       {
       lastword=CntStrWords(phra);
       }
    if (lastword<firstword)
       return(resul);
    for (k=firstword;k<=lastword;k++)
       {
       resul=resul+ReturnNthWord(phra,k);       
       if (k!=lastword)
          resul=resul+StringElement(WordsSeparatorsList,0);
       }
    return(resul);
    }

//***************************************************************
 // Splash functions
 //***************************************************************    

 void ClearSplash()
    {
    SplashText="";
    }
    
 void Splash(string s)
    {
    SplashText=SplashText+s;
    }
    
 void SplashLn(string s)
    {
    SplashText=SplashText+s+CRLF();
    }
    
 int SplashScreen(int buttons=EMPTY)
    {
    int res=MessageBox(SplashText,"",buttons);
    return(res);
    }
    
 //***************************************************************
 // Text display functions
 //***************************************************************    

 void ClearText()
    {
    string objname;
    int nobj=ObjectsTotal();    
    int i,isthere;
    i=1;
    while (true)      
      {
      objname="TextWritten"+AlltrimSpacesOnly(DoubleToStr(i,0));
      isthere=ObjectFind(objname);
      if (isthere!=-1)
         {
         ObjectDelete(objname);
         }
      else
         {
         break;
         }
      i=i+1;
      }
    TextStartX=_DefaultTextStartX;
    TextStartY=_DefaultTextStartY;
    TextNowY=_DefaultTextStartY;      
    ObjectsRedraw();
    LinesOfText=0;
    }

 void RefreshText()
    {
    string objname;
    int nobj=ObjectsTotal();    
    int i,isthere;
    i=1;
    while (true)      
      {
      objname="TextWritten"+AlltrimSpacesOnly(DoubleToStr(i,0));
      isthere=ObjectFind(objname);
      if (isthere!=-1)
         {
         ObjectSet(objname,OBJPROP_BACK,TextUnder);                 
         }
      else
         {
         break;
         }
      i=i+1;
      }
    TextStartX=_DefaultTextStartX;
    TextStartY=_DefaultTextStartY;
    TextNowY=_DefaultTextStartY;      
    ObjectsRedraw();
    LinesOfText=0;
    }
 
 
 
 
 void SetDisplayWindow(int handle)
    {
    TextDisplayWindow=handle;
    }
    
 void SetTextSpacing(int spacing)
    {
    TextFontSpacing=spacing;
    }

 void SetTextUnder(bool under)    
    {
    TextUnder=false;
    }
    
 void SetTextStart(int startx=0, int starty=0)
    {
    if (startx!=0) 
       {
       _DefaultTextStartX=startx; 
       }
    if (starty!=0) 
       {
       _DefaultTextStartX=starty;
       }           
    }            
 
    
 void SetTextParameters(string font,int size)
    {    
    TextFontName=font;
    TextFontSize=size;
    }
  
 void SetTextColor(int colour)
    {
    TextColor=colour;
    }
    
 void Write(string s)
    {
    string objname;
    if (LinesOfText==0)
       {
       objname="TextWritten"+AlltrimSpacesOnly(DoubleToStr(LinesOfText+1,0));              
       LinesOfText=LinesOfText+1;
       if (ObjectCreate(objname, OBJ_LABEL, TextDisplayWindow, 0, 0))
         {
         ObjectSet(objname,OBJPROP_XDISTANCE,TextStartX);         
         ObjectSet(objname,OBJPROP_YDISTANCE,TextNowY);  
         ObjectSet(objname,OBJPROP_BACK,TextUnder);                 
         ObjectSetText(objname,s,TextFontSize,TextFontName,TextColor);
         ObjectsRedraw();
         }    
       }
    else
       {
       objname="TextWritten"+AlltrimSpacesOnly(DoubleToStr(LinesOfText,0));              
       ObjectSet(objname,OBJPROP_XDISTANCE,TextStartX);
       ObjectSet(objname,OBJPROP_YDISTANCE,TextNowY);           
       ObjectSet(objname,OBJPROP_BACK,TextUnder); 
       ObjectSetText(objname,StringConcatenate(ObjectDescription(objname),s),TextFontSize,TextFontName,TextColor);
       ObjectsRedraw();
       }
   }
     
    
 
 void WriteLn(string s)
    {
    string objname;
    Write(s);
    LinesOfText=LinesOfText+1;
    objname="TextWritten"+AlltrimSpacesOnly(DoubleToStr(LinesOfText,0));              
    if (ObjectCreate(objname, OBJ_LABEL, TextDisplayWindow, 0, 0))
       {   
       ObjectSet(objname,OBJPROP_XDISTANCE,TextStartX);
       ObjectSet(objname,OBJPROP_YDISTANCE,TextNowY);       
       ObjectSet(objname,OBJPROP_BACK,TextUnder);     
       ObjectSetText(objname,"",TextFontSize,TextFontName,TextColor);
       TextNowY=TextNowY+TextFontSize+TextFontSpacing;
       }       
   }
   
   void Say(int row,string s)
       {
       string objname;
       if (row<=0)
          {
          row=1;
          }      
       if (row>LinesOfText)
          {
          for (int i=LinesOfText;i<row;i++) { WriteLn(""); }
          Write(s);
          }
       if (row==LinesOfText) 
          {
          objname="TextWritten"+AlltrimSpacesOnly(DoubleToStr(row,0));              
          ObjectSetText(objname,s,TextFontSize,TextFontName,TextColor);
          }
       if (row<LinesOfText)
          {
          objname="TextWritten"+AlltrimSpacesOnly(DoubleToStr(row,0));              
          ObjectSetText(objname,s,TextFontSize,TextFontName,TextColor);             
          }
       ObjectsRedraw();
       }
             
   void SayAppend(int row,string s)
       {
       string objname;
       if (row<=0)
          {
          row=1;
          }
       if (row>LinesOfText)
          {
          for (int i=LinesOfText;i<row;i++) { WriteLn(""); }
          Write(s);
          }
       if (row==LinesOfText) 
          {
          objname="TextWritten"+AlltrimSpacesOnly(DoubleToStr(row,0));              
          ObjectSetText(objname,ObjectDescription(objname)+s,TextFontSize,TextFontName,TextColor);
          ObjectSet(objname,OBJPROP_BACK,TextUnder); 
          }
       if (row<LinesOfText)
          {
          objname="TextWritten"+AlltrimSpacesOnly(DoubleToStr(row,0));              
          ObjectSetText(objname,ObjectDescription(objname)+s,TextFontSize,TextFontName,TextColor);             
          ObjectSet(objname,OBJPROP_BACK,TextUnder); 
          }         
       }
                              

 
    

 //***************************************************************
 // Time & date functions
 //***************************************************************    
    
int PackTime(datetime d,int yearbase)   
    {
    int a,th;
    a=TimeYear(d)-yearbase;
    a=a*366;
    a=a+TimeDayOfYear(d)-1;
    a=a*24;
    a=a+TimeHour(d)-1;
    a=a*60;
    a=a+TimeMinute(d)-1;
    a=a*60;
    a=a+TimeSeconds(d)-1;
    return(a);
    }
   

 //***************************************************************
 // Math functions 
 //***************************************************************
 
 int Round(double i)
    {
    string s;
    int res;
    s=DoubleToStr(i,0);
    res=StrToInteger(s);
    return(res);
    }
    
 double RoundTo(double i,int digits)
    {
    double res;
    double mp;
    mp=MathPow(10,digits);
    res=Round(i*mp)/mp;
    return(res);
    }
    
 int Truncate(double a)
    {
    double res;
    if (a>=0)
       res=MathRound(MathFloor(a));
    else
       res=MathRound(MathCeil(a));
    return(res);
    }           
   
 int MathDiv(int a,int b)
    {
    int res;
    res=a-MathRound(MathMod(a,b));
    res=MathRound(res/b);
    return(res);
    }
 
 bool Between(double v,double a,double b)
    {
    double aa;
    double bb;
    bool res;
    aa=MathMin(a,b);
    bb=MathMax(a,b);
    if (v>=a&&v<=b)
       res=True;
    else 
       res=False;
    return(res);
    }
 
 int Sgn(double x) 
    {    
    int v; 
    if (x<0) 
       v=-1;
    if (x==0)
       v=0;
    if (x>0)
       v=1;
    return(v);
    }
    
 double DtoR(double x)
    {
    double res;
    res=(x*2*pi)/360;
    return(res);
    }

 double RtoD(double x)
    {
    double res;
    res=(360*x)/(2*pi);
    return(res);
    }

 double ATan2(double x,double y)
    {
    double res;
    if (MathAbs(x)==0)       
       res=pi/2;       
    else
       res=MathArctan(MathAbs(y)/MathAbs(x));                     
    if ((x>=0&&y>=0)) 
       return(res);
    if ((x<0&&y>=0))       
       {
       res=pi-res;
       return(res);
       }
    if ((x<0&&y<0))
       {
       res=pi+res;
       return(res);
       }
    if ((x>=0&&y<0))
       {
       res=2*pi-res;
       return(res);
       }
    res=0;
    return(res);
    }
 
              
 
 void DetLine(double x1,double y1,double x2,double y2,double &m,double &n)
    {
    m=(y2-y1)/(x2-x1);
    n=y1-m*x1;
    }
    
 void DetIntersection(double m1,double n1,double m2,double n2, double &x,double &y)
    {
    y=(m2*n1-m1*n2)/(m1-m2);
    x=(y-n1)/m1;
    }
    
 void TranslateLine(double m,double n,double x,double y,double &m2,double &n2)
    {
    double x1,y1,angle;
    angle=ATan2(1,m);
    x1=x+1;
    y1=y+MathTan(angle);
    DetLine(x,y,x1,y1,m2,n2);        
    }
    
 double Distance(double x1,double y1,double x2,double y2)
    {
    double res;
    res=MathSqrt( (x2-x1)*(x2-x1) + (y2-y1)*(y2-y1) );
    return(res);
    }
    
 void RotatePoint(double xp,double yp,double xc,double yc,double AngleDisplacement,double &newx,double &newy)
    {
    double angle_r,angle,r;
    angle_r=ATan2(xp-xc,yp-yc);
    r=(xp-xc)/MathCos(angle_r);
    angle=angle_r+DtoR(AngleDisplacement);
    newx=xc+r*MathCos(angle);
    newy=yc+r*MathSin(angle);
    }
                 
    
 //***************************************************************
 // Statistics functions 
 //***************************************************************
     
void StatParameters(double array[],int p1,int p2,double &avg,double &variance,double &stddev)
    {  
    double suma=0;
    int i;    
    if (p2==0)
       p2=ArrayRange(array,0)-1;        
    for (i=p1;i<=p2;i++)
      suma=suma+array[i];
    avg=suma/(p2-p1+1);
    suma=0;
    for (i=p1;i<=p2;i++)
      suma=suma+MathPow(array[i]-avg,2);
    variance=suma/(p2-p1+1);    
    stddev=MathSqrt(suma);
    return;
    }
     
    
 void LinReg(double x[],double y[],int p1,int p2,double &a,double &b,double &variance,double &stddev,bool UseY)
    {
    int n,i;
    double sumaxy,xmed,ymed,sumaxp,sumayp,sumax,sumay;
    n=MathMin(ArrayRange(x,0),ArrayRange(y,0))-1;
    if (p2==0)
       p2=n;    
    sumax=0;
    sumay=0;
    sumaxy=0;
    sumaxp=0;
    sumayp=0;
    for (i=p1;i<=p2;i++)    
       {
       if (UseY==True)
          {
          sumax=sumax+x[i];       
          sumay=sumay+y[i];
          sumaxy=sumaxy+x[i]*y[i];
          sumaxp=sumaxp+MathPow(x[i],2);
          sumayp=sumayp+MathPow(y[i],2);
          }          
       else
          {          
          sumax=sumax+i;
          sumay=sumay+((x[i]+y[i])/2);       
          sumaxy=sumaxy+i*((x[i]+y[i])/2);
          sumaxp=sumaxp+MathPow(i,2);
          sumayp=sumayp+MathPow(((x[i]+y[i])/2),2);
          }                           
       }
    xmed=sumax/(p2-p1+1);
    ymed=sumay/(p2-p1+1);
    b=(sumaxy-((p2-p1+1)*xmed*ymed)) / (sumaxp-(p2-p1+1)*xmed*xmed);
    a=ymed-b*xmed;
    variance=MathAbs( (a*sumay+b*sumaxy-(p2-p1+1)*ymed*ymed) / (sumayp-(p2-p1+1)*ymed*ymed) );    
    stddev=MathSqrt(MathAbs(variance));
    if (b<0)
       stddev=-stddev;    
    }        
 
 void ExpReg2(double x[],double y[],int p1,int p2,double &a,double &b,double &variance,double &stddev,bool UseY)
    {
    int n,i;
    double sumaxy,xmed,ymed,sumaxp,sumayp,sumax,sumay,suma;
    n=MathMin(ArrayRange(x,0),ArrayRange(y,0))-1;
    if (p2==0)
       p2=n;    
    sumax=0;
    sumay=0;
    sumaxy=0;
    sumaxp=0;
    sumayp=0;
    for (i=p1;i<=p2;i++)    
       {
       if (UseY==True)
          {
          sumax=sumax+MathLog(x[i]);       
          sumay=sumay+MathLog(y[i]);
          sumaxy=sumaxy+MathLog(x[i])*MathLog(y[i]);
          sumaxp=sumaxp+MathPow(MathLog(x[i]),2);
          sumayp=sumayp+MathPow(MathLog(y[i]),2);
          }          
       else
          {          
          sumax=sumax+i;
          sumay=sumay+MathLog((x[i]+y[i])/2);       
          sumaxy=sumaxy+i*MathLog((x[i]+y[i])/2);
          sumaxp=sumaxp+MathPow(i,2);
          sumayp=sumayp+MathPow(MathLog((x[i]+y[i])/2),2);
          }                           
       }
    xmed=sumax/(p2-p1+1);
    ymed=sumay/(p2-p1+1);
    b=(sumaxy-((p2-p1+1)*xmed*ymed)) / (sumaxp-(p2-p1+1)*xmed*xmed);
    a=ymed-b*xmed;
    suma=0;
    variance=0;
    stddev=0;
    if (UseY==false)
       {
       for (i=p1;i<=p2;i++)    
          suma=suma+MathPow( (x[i]+y[i])/2 - MathPow(base_e,b*i+a) , 2 );
       variance=suma/(p2-p1+1);
       stddev=MathSqrt(MathAbs(variance));
       }     
    }
 //***************************************************************
 // T.A. stuff
 //***************************************************************
 
 string Line(int time0,double price0,int time1,double price1,int colour)
    {
    string res;    
    string objname;    
    double price2;          
    datetime tim0,tim1,tim2; 
    tim0=Time[time0];
    tim1=Time[time1];
    //applying interpolation here
    tim2=Round(MathMin(tim0,tim1)+(MathMax(tim0,tim1)-MathMin(tim0,tim1))/2);//since datetime is the number of passed milliseconds, it is continuous    
    price2=(price0+price1)/2;
    //generating object name        
    objname=StringConcatenate("Line",AlltrimSpacesOnly(DoubleToStr(LinesDrawn+1,0)));
    res=objname;
    if(!ObjectCreate(objname, OBJ_TRIANGLE, 0, tim0,price0,tim2,price2,tim1,price1))
       {            
         res="";
         return(res);
       }
    else 
       {
       ObjectSet(objname,OBJPROP_COLOR,colour);  
       ObjectSet(objname,OBJPROP_BACK,false);       
       LinesDrawn=LinesDrawn+1;
       }
    return(res);    
    }
    
 
 double ComputeWidth(int RegMethod,double Ind1[],double Ind2[],int p1,int p2,
         double &m,double &n,double &variance,double &stddev,
         double &WidthHighs,double &WidthLows,int &MethodChosen)
    {
    double nlin,mlin,rlin,rplin,nexp,mexp,rexp,rpexp;
    double HighestHigh,LowestLow,yhighest,ylowest,MaxErrorHighs,MaxErrorLows,ynormal,Width;
    int phighest,plowest;
    int i,regm;
    HighestHigh=-999999999;
    LowestLow=999999999;    
    MaxErrorHighs=0;
    MaxErrorLows=0;
    regm=-1;
    if (RegMethod==LINEAR_REGRESSION)
       regm=LINEAR_REGRESSION;
    if (RegMethod==EXPONNENTIAL_REGRESSION2)
       regm=EXPONNENTIAL_REGRESSION2;
    if (regm==-1)
       regm=0;
    LinReg(Ind1,Ind2,p1,p2,nlin,mlin,rlin,rplin,False);
    ExpReg2(Ind1,Ind2,p1,p2,nexp,mexp,rexp,rpexp,False);
    if (regm==0)
       {
       if (rplin<rpexp)
          regm=LINEAR_REGRESSION;
       else
          regm=EXPONNENTIAL_REGRESSION2;
       }            
    for (i=p1;i<=p2;i++)
        {
        if (regm==LINEAR_REGRESSION)
          {
          ynormal=mlin*i+nlin;
          }
        else 
          {
          ynormal=MathPow(base_e,mexp*i+nexp);    
          }         
        if (Ind1[i]-ynormal>MaxErrorHighs||Ind2[i]-ynormal>MaxErrorHighs)
           {
           if (Ind1[i]-ynormal>MaxErrorHighs)
              {
              HighestHigh=Ind1[i];
              phighest=i;
              MaxErrorHighs=Ind1[i]-ynormal;
              }
           else
              {
              HighestHigh=Ind2[i];
              phighest=i;
              MaxErrorHighs=Ind2[i]-ynormal;
              }                            
           }
        if (ynormal-Ind2[i]>MaxErrorLows||ynormal-Ind1[i]>MaxErrorLows)
           {
           if (ynormal-Ind2[i]>MaxErrorLows)
              {
              LowestLow=Ind2[i];
              plowest=i;
              MaxErrorLows=ynormal-Ind2[i];
              }
           else 
              {
              LowestLow=Ind1[i];
              plowest=i;
              MaxErrorLows=ynormal-Ind1[i];              
              }
           }
    }//endfor
    if (regm==LINEAR_REGRESSION)
       {
       yhighest=mlin*phighest+nlin;
       ylowest=mlin*plowest+nlin;
       m=mlin;
       n=nlin;
       variance=rlin;
       stddev=rplin;
       }
    else 
       {
       yhighest=MathPow(base_e,mexp*phighest+nexp);    
       ylowest=MathPow(base_e,mexp*plowest+nexp);    
       m=mexp;
       n=nexp;              
       variance=rexp;
       stddev=rpexp;       
       }
    Width=MathMax(HighestHigh-yhighest,ylowest-LowestLow)*2;    
    WidthHighs=HighestHigh-yhighest;
    WidthLows=ylowest-LowestLow;
    MethodChosen=regm;
    return(Width);
    }

 //***************************************************************
 // Other functions
 //***************************************************************

bool ContractExists(string symbol)
  {
  int Gle;
  double data;
  bool res;
  data=MarketInfo(symbol,MODE_POINT);
  Gle=GetLastError();
  if (Gle!=ERR_UNKNOWN_SYMBOL)
     res=true;
  return(res);
  }

double ConvertCurrency(double pips,string contract,string currency,double forceprice,string &errormsg)
  {
  errormsg="OK";
  double res=0;
  double initpoints;
  string woncrrcy,crrcytest,final,appendix;
  int poscr=1;
  double pr;
  contract=AlltrimSpacesOnly(contract);
  currency=AlltrimSpacesOnly(currency);
  bool ctex=ContractExists(contract);
  if (ctex==false)
     {
     errormsg="INVALID CONTRACT.";
     return(res);
     }
  woncrrcy=SubstrBetween(contract,3,5);
  appendix="";
  if (StringLen(contract)>6)
     {
     appendix=SubstrBetween(contract,6,StringLen(contract));    
     }
  if (woncrrcy==currency)
     {     
     res=pips;
     return(res);
     }
  crrcytest=currency+woncrrcy;
  poscr=1;  
  if (ContractExists(StringConcatenate(crrcytest,appendix))==true)     
     final=StringConcatenate(crrcytest,appendix);
  else
     {
     crrcytest=StringConcatenate(woncrrcy,currency);
     poscr=2;
     if (ContractExists(StringConcatenate(crrcytest,appendix))==true)
        final=StringConcatenate(crrcytest,appendix);
     else
        {
        errormsg="CONVERSION IMPOSSIBLE."; // still, could be tried cascade conversions...
        return(res);
        }
     } 
  if (forceprice==0)
     {
     if (poscr==1)
        res=MarketInfo(contract,MODE_POINT)/MarketInfo(final,MODE_POINT)*pips/MarketInfo(final,MODE_ASK);
     else
        res=MarketInfo(contract,MODE_POINT)/MarketInfo(final,MODE_POINT)*pips*MarketInfo(final,MODE_BID);
     }       
  else
     {
     if (poscr==1)
        res=MarketInfo(contract,MODE_POINT)/MarketInfo(final,MODE_POINT)*pips/forceprice;
     else
        res=MarketInfo(contract,MODE_POINT)/MarketInfo(final,MODE_POINT)*pips*forceprice;
     }  
  return(res);
  }


  
 //***************************************************************
 // Translation functions
 //***************************************************************
   
string Bool2Str(int boo)
     {
     switch(boo)
       {
       case 0:
             return("False");       
       default:
             return("True");
       }
     }

string TimeFrame2Str(int timef)
     {
     if (timef==PERIOD_M1)
        return("M1");      
     if (timef==PERIOD_M5)
        return("M5");      
     if (timef==PERIOD_M15)
        return("M15");      
     if (timef==PERIOD_M30)
        return("M30");      
     if (timef==PERIOD_H1)
        return("H1");      
     if (timef==PERIOD_H4)
        return("H4");      
     if (timef==PERIOD_D1)
        return("D1");
     if (timef==PERIOD_W1)
        return("W1");
     if (timef==PERIOD_MN1)
        return("MN1");               
     return("Unknown");       
     }
     
     
int Str2TimeFrame(string timef)
     {
     if (timef=="M1")
        return(PERIOD_M1);      
     if (timef=="M5")
        return(PERIOD_M5);      
     if (timef=="M15")
        return(PERIOD_M15);      
     if (timef=="M30")
        return(PERIOD_M30);      
     if (timef=="H1")
        return(PERIOD_H1);      
     if (timef=="H4")
        return(PERIOD_H4);      
     if (timef=="D1")
        return(PERIOD_D1);
     if (timef=="W1")
        return(PERIOD_W1);
     if (timef=="MN1")
        return(PERIOD_MN1);               
     return(-1);       
     }
     
string OrderType2Str(int otype)
     {
     if (otype==OP_BUY)
        return("Buy");
     if (otype==OP_SELL)
        return("Sell");
     if (otype==OP_BUYLIMIT)
        return("Buy Limit");
     if (otype==OP_SELLLIMIT)
        return("Sell Limit");
     if (otype==OP_BUYSTOP)
        return("Buy Stop");
     if (otype==OP_SELLSTOP)
        return("Sell Stop");
     return("Unknown");
     }
        
     

void TranslateError(int errcode,string &ErrType,string &ErrText,string &ErrExplanation)
     {
     ErrType="Unknown type.";
     ErrText="Unknown error.";
     ErrExplanation="Unknown reason";                
     //no need to inquire why damn switch is not working
      if (errcode==ERR_NO_ERROR) 
         {
         ErrType="Server error.";
         ErrText="No error.";
         ErrExplanation="Trade operation succeeded."; 
         return;
         }         
      if (errcode==ERR_NO_RESULT) 
         {
         ErrType="Server error.";
         ErrText="No result.";
         ErrExplanation="OrderModify attempts to replace the values already set with the same values. One or more values must be changed, then modification attempt can be repeated.";
         return;
         }                  
      if (errcode==ERR_COMMON_ERROR) 
         {
         ErrType="Server error.";
         ErrText="Common error.";
         ErrExplanation="Common error. All attempts to trade must be stopped until reasons are clarified. Restart of operation system and client terminal will possibly be needed.";  
         return;
         }         
      if (errcode==ERR_INVALID_TRADE_PARAMETERS) 
         {
         ErrType="Server error.";
         ErrText="Invalid trade parameters.";
         ErrExplanation="Invalid parameters were passed to the trading function, for example, wrong symbol, unknown trade operation, negative slippage, non-existing ticket number, etc. The program logic must be changed."; 
         return;
         }    
      if (errcode==ERR_SERVER_BUSY) 
         {
         ErrType="Server error.";
         ErrText="Server busy.";
         ErrExplanation="Trade server is busy. The attempt can be repeated after a rather long period of time (over several minutes)."; 
         return;
         }         
      if (errcode==ERR_OLD_VERSION) 
         {
         ErrType="Server error.";
         ErrText="Old version.";
         ErrExplanation="Old version of the client terminal. The latest version of the client terminal must be installed."; 
         return;
         }
      if (errcode==ERR_NO_CONNECTION) 
         {
         ErrType="Server error."; 
         ErrText="No connection.";
         ErrExplanation="No connection to the trade server. It is necessary to make sure that connection has not been broken (for example, using the IsConnected function) and repeat the attempt after a certain period of time (over 5 seconds)."; 
         return;
         }
      if (errcode==ERR_TOO_FREQUENT_REQUESTS) 
         {
         ErrType="Server error.";
         ErrText="Too frequent request.";
         ErrExplanation="Requests are too frequent. The frequency of requesting must be reduced, the program logic must be changed."; 
         return;
         }    
      if (errcode==ERR_ACCOUNT_DISABLED) 
         {
         ErrType="Server error.";
         ErrText="Account disabled.";
         ErrExplanation="The account was disabled. All attempts to trade must be stopped."; 
         return;
         }
      if (errcode==ERR_INVALID_ACCOUNT) 
         {
         ErrType="Server error.";
         ErrText="Invalid account.";
         ErrExplanation="The account number is invalid. All attempts to trade must be stopped."; 
         return;
         }     
      if (errcode==ERR_TRADE_TIMEOUT) 
         {
         ErrType="Server error.";
         ErrText="Trade timeout.";
         ErrExplanation="Timeout for the trade has been reached. Before retry (at least, in 1-minute time),"+
         " it is necessary to make sure that trading operation has not really succeeded (a new position has not been opened, or the existing order has not been modified or deleted, or the existing position has not been closed)."; 
         return;         
         }
      if (errcode==ERR_INVALID_PRICE) 
         {
         ErrType="Server error.";
         ErrText="Invalid price.";
         ErrExplanation="Invalid bid or ask price, perhaps, unnormalized price. After 5-second (or more) "+
         "delay, it is necessary to refresh data using the RefreshRates function and make a retry. If the error does not disappear, all attempts to trade must be stopped, the program logic must be changed."; 
         return;
         }
      if (errcode==ERR_INVALID_STOPS) 
         {
         ErrType="Server error.";
         ErrText="Invalid stops.";
         ErrExplanation="Stops are too close, or prices are ill-calculated or unnormalized (or in the open "+
         "price of a pending order). The attempt can be repeated only if the error occurred due to the price "+
         "obsolescense. After 5-second (or more) delay, it is necessary to refresh data using the RefreshRates function and make a retry. If the error does not disappear, all attempts to trade must be stopped, the program logic must be changed.";
         return;
         }         
      if (errcode==ERR_INVALID_TRADE_VOLUME) 
         {
         ErrType="Server error.";
         ErrText="Invalid trade volume:";
         ErrExplanation="Invalid trade volume, error in the volume granularity. All attempts to trade must be stopped, and the program logic must be changed."; 
         return;         
         }
      if (errcode==ERR_MARKET_CLOSED) 
         {
         ErrType="Server error.";
         ErrText="Market closed.";
         ErrExplanation="Market is closed. The attempt can be repeated after a rather long period of time (over several minutes)."; 
         return;
         }     
      if (errcode==ERR_TRADE_DISABLED) 
         {
         ErrType="Server error.";
         ErrText="Trade disabled.";
         ErrExplanation="Trade is disabled. All attempts to trade must be stopped."; 
         return;         
         }
      if (errcode==ERR_NOT_ENOUGH_MONEY) 
         {      
         ErrType="Server error.";
         ErrText="Not enough money.";
         ErrExplanation="Not enough money to make an operation. The trade with the same parameters must not be repeated."+         
         " After 5-second (or more) delay, the attempt can be repeated with a smaller volume, but it is necessary to make sure that there is enough money to complete the operation."; 
         return;
         }         
      if (errcode==ERR_PRICE_CHANGED) 
         {
         ErrType="Server error.";
         ErrText="Price changed.";
         ErrExplanation="The price has changed. The data can be refreshed without any delay using the RefreshRates function and make a retry."; 
         return;
         }         
      if (errcode==ERR_OFF_QUOTES) 
         {
         ErrType="Server error.";
         ErrText="No quotes.";
         ErrExplanation="No quotes. The broker has not supplied with prices or refused, for any reason (for example, no prices"+
         " at the session start, unconfirmed prices, fast market). After 5-second (or more) delay, it is necessary to refresh data using the RefreshRates function and make a retry."; 
         return;
         }         
      if (errcode==ERR_REQUOTE) 
         {         
         ErrType="Server error.";
         ErrText="Requote.";
         ErrExplanation="The requested price has become out of date (more than 5 ticks have income) or bid and ask prices"+
         " have been mixed up. The data can be refreshed without any delay using the RefreshRates function and make a retry. If the error does not disappear, all attempts to trade must be stopped, the program logic must be changed."; 
         return;
         }         
      if (errcode==ERR_ORDER_LOCKED) 
         {
         ErrType="Server error.";
         ErrText="Order locked.";
         ErrExplanation="The order has been locked and under processing. All attempts to make trading operations must be stopped, and the program logic must be changed."; 
         return;
         }         
      if (errcode==ERR_LONG_POSITIONS_ONLY_ALLOWED) 
         {
         ErrType="Server error.";
         ErrText="Long positions only allowed.";
         ErrExplanation="Only buying operation is allowed. The SELL operation must not be repeated."; 
         return;         
         }
      if (errcode==ERR_TOO_MANY_REQUESTS) 
         {
         ErrType="Server error.";
         ErrText="Too many many requests.";
         ErrExplanation="Too many requests. The frequency of requesting must be reduced, the program logic must be changed."; 
         return;
         }         
      if (errcode==ERR_TRADE_MODIFY_DENIED) 
         {
         ErrType="Server error.";
         ErrText="Trade modify denied.";
         ErrExplanation="Modifying has been denied since the order is too close to market and locked for possible soon execution. The data can be refreshed after more than 15 seconds using the RefreshRates function, and a retry can be made."; 
         return;
         }         
      if (errcode==ERR_TRADE_CONTEXT_BUSY) 
         {
         ErrType="Server error.";
         ErrText="Trade context busy.";
         ErrExplanation="The trade thread is busy. Retry only after the IsTradeAllowed function has returned TRUE."; 
         return;
         }         
      if (errcode==ERR_TRADE_EXPIRATION_DENIED) 
         {
         ErrType="Server error";
         ErrText="Trade expiration denied.";
         ErrExplanation="The use of pending order expiration date has been denied by the broker. The operation can only be repeated if the expiration parameter has been zeroized."; 
         return;
         }         
      if (errcode>=ERR_NO_MQLERROR)
         {
         ErrType="Runtime error.";
         ErrExplanation="Not available.";
         if (errcode==ERR_NO_MQLERROR)
            {
            ErrText="No error.";
            ErrExplanation="Not needed.";
            return;
            }
         if (errcode==4001)
            {
            ErrText="Wrong function pointer.";
            return;
            }          
         if (errcode==4002)
            {
            ErrText="Array index is out of range.";
            return;
            }          
         if (errcode==4003)
            {
            ErrText="No memory for function call stack.";
            return;
            }          
         if (errcode==4004)
            {
            ErrText="Recursive stack overflow.";
            return;
            }          
         if (errcode==4005)
            {
            ErrText="Not enough stack for parameter.";
            return;
            }          
         if (errcode==4006)
            {
            ErrText="No memory for parameter string.";
            return;
            }          
         if (errcode==4007)
            {
            ErrText="No memory for temp string.";
            return;
            }          
         if (errcode==4008)
            {
            ErrText="Not initialized string.";
            return;
            }          
         if (errcode==4009)
            {
            ErrText="Not initialized string in array.";
            return;
            }      
         if (errcode==4010)
            {
            ErrText="No memory for array string.";
            return;
            }
         if (errcode==4011)
            {
            ErrText="Too long string.";
            return;
            }          
         if (errcode==4012)
            {
            ErrText="Remainder from zero divide.";
            return;
            }          
         if (errcode==4013)
            {
            ErrText="Zero divide.";
            return;
            }          
         if (errcode==4014)
            {
            ErrText="Unknown command.";
            return;
            }          
         if (errcode==4015)
            {
            ErrText="Wrong jump (never generated error).";
            return;
            }          
         if (errcode==4016)
            {
            ErrText="Not initialized array.";
            return;
            }          
         if (errcode==4017)
            {
            ErrText="DLL calls are not allowed.";
            return;
            }          
         if (errcode==4018)
            {
            ErrText="Cannot load library.";
            return;
            }          
         if (errcode==4019)
            {
            ErrText="Cannot call function.";
            return;
            }                      
         if (errcode==4020)
            {
            ErrText="Expert function calls are not allowed.";
            return;
            }
         if (errcode==4021)
            {
            ErrText="Not enough memory for temp string returned from function.";
            return;
            }          
         if (errcode==4022)
            {
            ErrText="System is busy (never generated error).";
            return;
            }
         if (errcode==4050)
            {
            ErrText="Invalid function for parameters count.";
            return;
            }
         if (errcode==4051)
            {
            ErrText="Invalid function parameter value.";
            return;
            }          
         if (errcode==4052)
            {
            ErrText="String function internal error.";
            return;
            }          
         if (errcode==4053)
            {
            ErrText="Some array error.";
            return;
            }          
         if (errcode==4054)
            {
            ErrText="Incorrect series array using.";            
            return;
            }          
         if (errcode==4055)
            {
            ErrText="Custom indicator error.";
            return;
            }          
         if (errcode==4056)
            {
            ErrText="Global variables processing error.";
            return;
            }          
         if (errcode==4057)
            {
            ErrText="Global variables processing error.";
            return;
            }          
         if (errcode==4058)
            {
            ErrText="Global variable not found.";
            return;
            }          
         if (errcode==4059)
            {
            ErrText="Function is not allowed in testing mode."; 
            return;
            }               
         if (errcode==4060)
            {
            ErrText="Function is not confirmed.";
            return;
            }
         if (errcode==4061)
            {
            ErrText="Send mail error.";
            return;
            }          
         if (errcode==4062)
            {
            ErrText="String parameter expected.";
            return;
            }          
         if (errcode==4063)
            {
            ErrText="Integer parameter expected.";
            return;
            }          
         if (errcode==4064)
            {
            ErrText="Double parameter expected.";            
            return;
            }          
         if (errcode==4065)
            {
            ErrText="Array as parameter expected.";
            return;
            }          
         if (errcode==4066)
            {
            ErrText="Requested history data in updating state.";
            return;
            }          
         if (errcode==4067)
            {
            ErrText="Some error in trading function.";
            return;
            }               
         if (errcode==4099)
            {
            ErrText="End of file.";
            return;
            }                                    
         if (errcode==4100)
            {
            ErrText="Some file error.";
            return;
            }
         if (errcode==4101)
            {
            ErrText="Wrong file name.";
            return;
            }          
         if (errcode==4102)
            {
            ErrText="Too many opened files.";
            return;
            }          
         if (errcode==4103)
            {
            ErrText="Cannot open file.";
            return;
            }          
         if (errcode==4104)
            {
            ErrText="Incompatible access to a file.";            
            return;
            }          
         if (errcode==4105)
            {
            ErrText="No order selected.";
            return;
            }          
         if (errcode==4106)
            {
            ErrText="Unknown symbol.";
            return;
            }          
         if (errcode==4107)
            {
            ErrText="Invalid price.";
            return;
            }          
         if (errcode==4108)
            {
            ErrText="Invalid ticket.";
            return;
            }          
         if (errcode==4109)
            {
            ErrText="Trade is not allowed."; 
            return;
            }               
         if (errcode==4110)
            {
            ErrText="Longs are not allowed.";
            return;
            }
         if (errcode==4111)
            {
            ErrText="Shorts are not allowed";
            return;
            }                      
         if (errcode==4200)
            {
            ErrText="Object exists already.";
            return;
            }
         if (errcode==4201)
            {
            ErrText="Unknown object property.";
            return;
            }          
         if (errcode==4202)
            {
            ErrText="Object does not exist.";
            return;
            }          
         if (errcode==4203)
            {
            ErrText="Unknown object type.";
            return;
            }          
         if (errcode==4204)
            {
            ErrText="No object name.";            
            return;
            }          
         if (errcode==4205)
            {
            ErrText="Object coordinates error.";
            return;
            }          
         if (errcode==4206)
            {
            ErrText="No specified subwindow.";
            return;
            }          
         if (errcode==4207)
            {
            ErrText="Some error in object function.";
            return;
            }                              
         ErrText="Unknown error.";
         ErrExplanation="Unknown reason.";
         return;          
         }                                      
    return;         
    } //end function
                
                   
               
             
                         


