//+------------------------------------------------------------------+
//|                                                    FPITrader.mq4 |
//|                                                 Bogdan Caramalac |
//|                                     mailto:fxeconomist@yahoo.com |
//+------------------------------------------------------------------+
//
// by an original idea of Michal Kreslik
// 
#property copyright "Bogdan Caramalac"
#property link      "mailto:fxeconomist@yahoo.com"


extern string Appendix="m";
extern double FPI_LO=0.9997;
extern double FPI_HI=1.0003;
extern int Slippage=3;
extern int MarginUsage=75;
extern int DisplayFromRow=16;

#include <winuser32.mqh>
#include <bclib.mq4>

#define BSS 0
#define SBB 1
       
string PossibleRings[4,2]={
       "EURUSD  EURGBP  GBPUSD","*//",
       "EURCHF  EURUSD  USDCHF","/**",
       "EURJPY  EURGBP  GBPJPY","/**",
       "EURJPY  EURUSD  USDJPY","/**"};                       

string Currencies[8]={"EUR","USD","GBP","JPY","CHF","AUD","CAD","NZD"};       
       
int RingsCount=0;
string RealRings[10,4];
int RealMagics[10];
bool SomethingNotOk=False;


//***********************************************************************
//                    Currency coding functions
//***********************************************************************

int CrrcyToInt(string crrcy)
    {
    string uc;
    int i;
    uc=Upper(crrcy);
    for (i=0;i<8;i++)
       {
       if (Currencies[i]==crrcy||Currencies[i]==uc)
         {
         return(i);       
         }    
       }
    return(-1);
    }   
    
string IntToCrrcy(int in)
    {
    return(Currencies[in]);
    }
        
    
int ContractToInt(string contract)    
    {
    int res,c1,c2;
    string cwork,aphere;
    aphere="";
    if (StringLen(contract)>6)    
       {
       aphere=SubstrBetween(contract,6,StringLen(contract));
       cwork=SubstrBetween(contract,1,6);
       }
    else    
       cwork=contract;
    c1=CrrcyToInt(SubstrBetween(cwork,0,2));
    c2=CrrcyToInt(SubstrBetween(cwork,3,5));
    res=c1*8+c2;
    return(res);
    }
    
string IntToContract(int in,string ap)
    {
    int c1,c2;
    string res;
    c2=Round(MathMod(in,8));
    c1=Round(MathDiv(in,8));
    res=Currencies[c1]+Currencies[c2]+ap;
    return(res);
    }

int PositionToMagic(string c1,string c2, string c3)
    {
    int res;
    res=ContractToInt(c1);
    res=res*64;
    res=res+ContractToInt(c2);
    res=res*64;
    res=res+ContractToInt(c3);
    return(res);
    }
    
void MagicToPosition(int magic,string &c1,string &c2,string &c3)
    {
    int r1,r2,r3;
    r3=MathRound(MathMod(magic,64));
    magic=MathDiv(magic,64);
    r2=MathRound(MathMod(magic,64));
    magic=MathDiv(magic,64);
    r1=magic;
    c1=IntToContract(r1,Appendix);
    c2=IntToContract(r2,Appendix);
    c3=IntToContract(r3,Appendix);       
    }
    
int GetRingPosition(int magic)
    {
    int res=-1;
    int i;
    for (i=0;i<RingsCount;i++)
       {
       if (RealMagics[i]==magic)
          {
          res=i;
          break;
          }
       }
    return(res);
    }

//***********************************************************************
//   some miscellaneous calculations
//***********************************************************************
string Crr(string contract,int whichone)
  {
  string res;
  if (whichone==1)
     {
     res=SubstrBetween(contract,0,2);  
     }
  else
     {
     res=SubstrBetween(contract,3,5);
     }
  return(res);
  }
  

string GotOrPaid(int optype,double volume,string contract,int whichone)
  {
  string res;
  if (optype==OP_BUY&&whichone==1)
     {
     res=StringConcatenate("we got ",DoubleToStr(volume,0)," ",Crr(contract,whichone));
     }
  if (optype==OP_BUY&&whichone==2)
     {
     res=StringConcatenate("we paid ",DoubleToStr(volume,0)," ",Crr(contract,whichone));
     }
  if (optype==OP_SELL&&whichone==2)
     {
     res=StringConcatenate("we got ",DoubleToStr(volume,0)," ",Crr(contract,whichone));
     }
  if (optype==OP_SELL&&whichone==1)
     {
     res=StringConcatenate("we paid ",DoubleToStr(volume,0)," ",Crr(contract,whichone));
     }
  return(res);     
  }
  
double PartialHedge(double howmuch, double ApplyIfPositive, double LotsOnPositive, double ApplyIfNegative, double LotsOnNegative)
  {
  double res;
  if (howmuch>=0)
     res=(howmuch/LotsOnPositive)*ApplyIfPositive;
  else
     res=-MathAbs((howmuch/LotsOnNegative)*ApplyIfNegative);
  return(res);
  }


//***********************************************************************

int RevRingType(int RingType)
    {
    int res;
    if (RingType==BSS)
       res=SBB;
    if (RingType==SBB)
       res=BSS;
    return(res);
    }


double CalculateRingFPI(int RingPos,int RingType)
  {
  double p1,p2,p3,res;
  string s1,s2,s3;
  if (RingType==BSS)
     {
     p1=MarketInfo(RealRings[RingPos,0],MODE_ASK);
     p2=MarketInfo(RealRings[RingPos,1],MODE_BID);
     p3=MarketInfo(RealRings[RingPos,2],MODE_BID);
     }
  if (RingType==SBB)
     {
     p1=MarketInfo(RealRings[RingPos,0],MODE_ASK);
     p2=MarketInfo(RealRings[RingPos,1],MODE_BID);
     p3=MarketInfo(RealRings[RingPos,2],MODE_BID);
     }
  s1=StringElement(RealRings[RingPos,3],0);
  s2=StringElement(RealRings[RingPos,3],1);
  s3=StringElement(RealRings[RingPos,3],2);
  res=1;
  if (s1=="*")
     res=res*p1;
  else
     res=res/p1;
  if (s2=="*")
     res=res*p2;
  else
     res=res/p2;
  if (s3=="*")
     res=res*p3;
  else
     res=res/p3;
  return(res);
  }  
  

void GetActiveRing(int &ActiveRing, int &RingType)
  {
  ActiveRing=-1;
  RingType=-1;
  int magic;
  int ntrades=OrdersTotal(); 
  if (ntrades!=0)
    {
    OrderSelect(0,SELECT_BY_POS,MODE_TRADES);
    magic=OrderMagicNumber();
    RingType=MathRound(MathMod(magic,2));
    ActiveRing=MathDiv(magic,2);
    }
  }

void GetPositionData(int Grp, int RingType, double &FirstLot, double &SecondLot, double &r1, double &r2, double &r3, double &pro1, double &pro2, double &pro3)
  {
  int ntrades=OrdersTotal(); 
  int mustbemagic=RealMagics[Grp]*2+RingType;
  int magic,i,j;
  for (i=0;i<3;i++)
    {
    for (j=0;j<ntrades;j++)
       {
       OrderSelect(j,SELECT_BY_POS,MODE_TRADES);
       magic=OrderMagicNumber();
       if (magic==mustbemagic)
          {
          if (OrderSymbol()==RealRings[Grp,i])
             {
             if (i==0)
                {
                r1=OrderOpenPrice();
                FirstLot=OrderLots();
                pro1=OrderProfit();
                }
             if (i==1)
                {
                r2=OrderOpenPrice();//lot is FirstLot, too              
                pro2=OrderProfit();
                }
             if (i==2)
                {
                r3=OrderOpenPrice();                
                SecondLot=OrderLots();
                pro3=OrderProfit();
                }             
             }//if (OrderSymbol()==RealRings[Grp,i])   
          }//if (magic==mustbemagic)
       }//for (j=0;j<ntrades;j++)
    }//for (i=0;i<2;i++)
  return; 
  }


void Position_Create(int Position, int RingType)
  {
  double available,r1,r2,r3;
  double first_volume,second_volume,second_volume_to_minus,second_volume_to_plus,dminus,dplus,required;
  double pricecl;  
  string c1,c2,c3,etype,etext,eexpl;
  int op1,op2,op3,magic,ticket1,ticket2,ticket3,Gle;
  c1=RealRings[Position,0];
  c2=RealRings[Position,1];
  c3=RealRings[Position,2];
  magic=RealMagics[Position]*2+RingType;
  available=AccountLeverage()*MathRound(AccountEquity()*MarginUsage/100);
  if (RingType==BSS)
     {
     r1=MarketInfo(c1,MODE_ASK);
     op1=OP_BUY;
     r2=MarketInfo(c2,MODE_BID);
     op2=OP_SELL;
     r3=MarketInfo(c3,MODE_BID);     
     op3=OP_SELL;
     }
  else
     {
     r1=MarketInfo(c1,MODE_BID);
     op1=OP_SELL;
     r2=MarketInfo(c2,MODE_ASK);
     op2=OP_BUY;
     r3=MarketInfo(c3,MODE_ASK);     
     op3=OP_BUY;
     }  
  // example
  // [DIRECTION] EURJPY BIG_VOLUME * R1 = V1
  // [REV.DIR.]  EURUSD BIG_VOLUME * R2 = V2
  // [REV.DIR.]  USDJPY V1 * R3 = V3     
  first_volume=MathRound(available/(2+r2));    //maximum for big_volume
  first_volume=1000*Truncate(first_volume/1000); //cutting garbage
  required=first_volume*r2;
  second_volume_to_minus=Truncate(required/1000)*1000;
  second_volume_to_plus=(Truncate(required/1000)+1)*1000;
  dminus=required-second_volume_to_minus;
  dplus=second_volume_to_plus-required;
  if (dminus<=dplus)
     second_volume=MathRound(second_volume_to_minus/1000)*0.01;  //converting to lots
  else
     second_volume=MathRound(second_volume_to_plus/1000)*0.01;   //converting to lots
  first_volume=MathRound(first_volume/1000)*0.01;                  //converting to lots  
  
  if (SomethingNotOk==false&&IsTradeAllowed()==True)
  {
  while (IsTradeContextBusy()==true)
     { // waste time
     }   
  ticket1=OrderSend(c1,op1,first_volume,r1,Slippage,0,0,"",magic,0,Blue);  
  if (ticket1==-1)
     {
     Gle=GetLastError();
     TranslateError(Gle,etype,etext,eexpl);           
     SetTextColor(Red);
     WriteLn("Encountered error");
     WriteLn(etext);
     WriteLn("while attempting to trade "+DoubleToStr(first_volume,2)+" lots on "+c1);
     WriteLn("Shutting down.");
     SomethingNotOk=True;
     SetTextColor(White);
     return;
     }
  while (IsTradeContextBusy()==True)
     { // waste time
     }      
  ticket2=OrderSend(c2,op2,first_volume,r2,Slippage,0,0,"",magic,0,Blue);
  if (ticket2==-1)
     {
     Gle=GetLastError();
     TranslateError(Gle,etype,etext,eexpl);      
     OrderSelect(ticket1,SELECT_BY_TICKET);
     if (OrderType()==OP_BUY)
        pricecl=MarketInfo(OrderSymbol(),MODE_BID);
     if (OrderType()==OP_SELL)
        pricecl=MarketInfo(OrderSymbol(),MODE_ASK);                     
     OrderClose(OrderTicket(),OrderLots(),pricecl,Slippage,Red);     
     SetTextColor(Red);
     WriteLn("Encountered error");
     WriteLn(etext);
     WriteLn("while attempting to trade "+DoubleToStr(first_volume,2)+" lots on "+c2);
     WriteLn("Removed trade on "+c1);
     WriteLn("Shutting down.");
     SomethingNotOk=True;
     SetTextColor(White);
     return;
     }
  while (IsTradeContextBusy()==True)
     { // waste time
     }        
  ticket3=OrderSend(c3,op3,second_volume,r3,Slippage,0,0,"",magic,0,Blue);
  if (ticket3==-1)
     {
     Gle=GetLastError();
     TranslateError(Gle,etype,etext,eexpl);      
     OrderSelect(ticket1,SELECT_BY_TICKET);
     if (OrderType()==OP_BUY)
        pricecl=MarketInfo(OrderSymbol(),MODE_BID);
     if (OrderType()==OP_SELL)
        pricecl=MarketInfo(OrderSymbol(),MODE_ASK);                     
     OrderClose(OrderTicket(),OrderLots(),pricecl,Slippage,Red);     
     OrderSelect(ticket2,SELECT_BY_TICKET);
     if (OrderType()==OP_BUY)
        pricecl=MarketInfo(OrderSymbol(),MODE_BID);
     if (OrderType()==OP_SELL)
        pricecl=MarketInfo(OrderSymbol(),MODE_ASK);                     
     OrderClose(OrderTicket(),OrderLots(),pricecl,Slippage,Red);          
     SetTextColor(Red);
     WriteLn("Encountered error");
     WriteLn(etext);
     WriteLn("while attempting to trade "+DoubleToStr(second_volume,2)+" lots on "+c3);
     WriteLn("Removed trade on "+c1);
     WriteLn("Removed trade on "+c2);
     WriteLn("Shutting down.");
     SomethingNotOk=True;
     SetTextColor(White);
     return;
     }      
  }//somethingnotok==false
  }
  
void Position_Destroy(int ActiveRing)
  {  
  int magic,i;
  int RingType;    
  double pricecl;
  string os;
  int ntrades=OrdersTotal(); 
  if (ntrades!=0)
    {
    for (i=0;i<ntrades;i++)
       {
       OrderSelect(0,SELECT_BY_POS,MODE_TRADES);
       magic=OrderMagicNumber();
       RingType=MathRound(MathMod(magic,2));
       if (MathDiv(magic,2)==ActiveRing)
         {
         os=OrderSymbol();
         if (OrderType()==OP_BUY)
            pricecl=MarketInfo(os,MODE_BID);
         if (OrderType()==OP_SELL)
            pricecl=MarketInfo(os,MODE_ASK);                     
         OrderClose(OrderTicket(),OrderLots(),pricecl,Slippage,Green);                
         SetTextColor(Green);
         Say(30,"Removed trade on "+os);
         SetTextColor(White);         
         }
       }       
    }
  }


//+------------------------------------------------------------------+
//| expert MAIN function                                             |
//+------------------------------------------------------------------+
bool Main()
  {
   int iRing=0;
   int grp=-1;
   int ActiveRing,RingType;   
   int cring;
   double FirstLot,SecondLot,r1,r2,r3,First,Second;
   double RingBSS,RingSBB,CloseFPI;
   double MinRingCoef;
   double opro1,opro2,opro3;
   int MinRingIndex,MinRingType,op1,op2,op3;
   string c1,c2,c3;
   //Active Ring Check
   GetActiveRing(ActiveRing,RingType);     
   if (ActiveRing!=-1)
      {
      grp=GetRingPosition(ActiveRing);      
      if (RingType==BSS)
         {
         op1=OP_BUY;
         op2=OP_SELL;
         op3=OP_SELL;
         }
      else
         {
         op1=OP_SELL;
         op2=OP_BUY;
         op3=OP_BUY;
         }
      c1=RealRings[grp,0];
      c2=RealRings[grp,1];
      c3=RealRings[grp,2];
      GetPositionData(grp,RingType,FirstLot,SecondLot,r1,r2,r3,opro1,opro2,opro3);
      First=FirstLot/0.01*1000;
      Second=SecondLot/0.01*1000;
      SetTextColor(Yellow);
      Say(1,"Position active on");                
      if (RingType==BSS)
         {           
         Say(2,StringConcatenate("Ring BUY  ",RealRings[grp,0],"  SELL ",RealRings[grp,1],"  SELL ",RealRings[grp,2]));
         }
      else
         {
         Say(2,StringConcatenate("Ring SELL  ",RealRings[grp,0],"  BUY ",RealRings[grp,1],"  BUY ",RealRings[grp,2]));
         }         
      CloseFPI=CalculateRingFPI(grp,RevRingType(RingType));
      Say(4,StringConcatenate("Close FPI= ",DoubleToStr(CloseFPI,4)));            
      Say(6,StringConcatenate("Current result : ",DoubleToStr(opro1+opro2+opro3,2)));      
      Say(8,"Exposure");      
      Say(9,StringConcatenate(DoubleToStr(MathAbs(First*r2-Second),0)," ",Crr(c3,1)," produced ",DoubleToStr( PartialHedge(First*r2-Second,opro2,First*r2,opro3,Second),2)  ));
      Say(10,StringConcatenate(DoubleToStr(MathAbs(First*r1-Second*r3),0)," ",Crr(c3,2)," produced ",DoubleToStr( PartialHedge(First*r1-Second*r3,opro1,First*r1,opro3,Second*r3),2) ));      
      Say(12,"Exposure under <Impeccable Hedge> assumption");
      Say(13,StringConcatenate(DoubleToStr(MathAbs(First*r1-First*r2*r3),0)," ",Crr(c3,2)," produced ",DoubleToStr( PartialHedge(First*r1-First*r2*r3,opro1,First*r1,opro3,First*r2*r3),2) ));
      SetTextColor(White);        
      if (CloseFPI>=FPI_HI)
         { 
         Position_Destroy(ActiveRing);
         }        
      }     
   //Cycle possible rings         
   MinRingCoef=9999999999;
   MinRingIndex=-1;
   MinRingType=2;
   cring=0;
   for (iRing=0;iRing<RingsCount;iRing++)
     {          
     RingBSS=CalculateRingFPI(iRing,BSS);
     RingSBB=CalculateRingFPI(iRing,SBB);
     Say(DisplayFromRow+cring,StringConcatenate("Ring BUY  ",RealRings[iRing,0],"  SELL ",RealRings[iRing,1],"  SELL ",RealRings[iRing,2],"  FPI = ",DoubleToStr(RingBSS,4)));
     cring=cring+1;
     Say(DisplayFromRow+cring,StringConcatenate("Ring SELL ",RealRings[iRing,0],"  BUY  ",RealRings[iRing,1],"  BUY  ",RealRings[iRing,2],"  FPI = ",DoubleToStr(RingSBB,4)));
     cring=cring+1;
     if (RingBSS<MinRingCoef)
        {
        MinRingCoef=RingBSS;
        MinRingIndex=iRing;
        MinRingType=BSS;
        }
     if (RingSBB<MinRingCoef)
        {
        MinRingCoef=RingBSS;
        MinRingIndex=iRing;
        MinRingType=SBB;
        }        
     }           
   
   //Entering position on a ring, if available
   GetActiveRing(ActiveRing,RingType);//cause ring check might close current ring
   if (MinRingCoef<=FPI_LO&&ActiveRing==-1)
      {
      Position_Create(MinRingIndex,MinRingType);
      if (SomethingNotOk==true)
         {
         return(false);
         }
     }   
   
   
   return(true);
  }
//+------------------------------------------------------------------+  


  
    

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
  int rels=Round(ArraySize(PossibleRings)/2);
  int i;
  int rcount;
  string c1,c2,c3,op;
  bool cex1,cex2,cex3,opok;
  rcount=0;
  ClearText();  
  for (i=0;i<rels;i++)
     {
     opok=false;
     c1=ReturnNthWord(PossibleRings[i,0],1);
     c2=ReturnNthWord(PossibleRings[i,0],2);
     c3=ReturnNthWord(PossibleRings[i,0],3);
     op=PossibleRings[i,1];
     cex1=ContractExists(StringConcatenate(c1,Appendix));
     cex2=ContractExists(StringConcatenate(c2,Appendix));
     cex3=ContractExists(StringConcatenate(c3,Appendix));
 
     if (cex1==True&&cex2==True&&cex3==True)
        {
        RealRings[rcount,0]=StringConcatenate(c1,Appendix);
        RealRings[rcount,1]=StringConcatenate(c2,Appendix);
        RealRings[rcount,2]=StringConcatenate(c3,Appendix);
        RealRings[rcount,3]=op;        
        RealMagics[rcount]=PositionToMagic(c1,c2,c3);
        rcount=rcount+1;
        }
     else
        {
        ClearSplash();        
        if (cex1==false)
           SplashLn(StringConcatenate("Contract not found :",StringConcatenate(c1,Appendix))); 
        if (cex2==false)                      
           SplashLn(StringConcatenate("Contract not found :",StringConcatenate(c2,Appendix)));
        if (cex3==false)                      
           SplashLn(StringConcatenate("Contract not found :",StringConcatenate(c3,Appendix))); 
        SplashScreen();
        return(1);         
        }   
     } 
  RingsCount=rcount;
  for (i=1;i<=23;i++)
      {
      WriteLn("");
      }   
  
  //*****************************************************************************
            
  while (true==true)
    { 
    bool res;
    RefreshRates();
    Main();     
    if (SomethingNotOk==True)
       return;
    }              
  
  return(0);
  }
  

//+------------------------------------------------------------------+
//| expert LOOPING function                                          |
//+------------------------------------------------------------------+  
int ContinueLooping()
   {
   while (true==true)
    { 
    RefreshRates();
    Main();     
    if (SomethingNotOk==True)
       return;    
    }            
      
   return(0);      
   }  

  
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
  if (SomethingNotOk==true)
     return(1);  
  ContinueLooping();
  if (SomethingNotOk==True)
     return(1);
  return(0);
  }  


//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
   return(0);
  }

