//+------------------------------------------------------------------+
//|                                                      fpitest.mq4 |
//|                                                 Bogdan Caramalac |
//|                                     mailto:fxeconomist@yahoo.com |
//+------------------------------------------------------------------+
//
// by an original idea of Michal Kreslik
// 
#property copyright "Bogdan Caramalac"
#property link      "mailto:fxeconomist@yahoo.com"



extern string Appendix="m";
extern double FPI_LO=0.9997;
extern double FPI_HI=1.0003;
extern int Slippage=3;
extern int MarginUsage=75;

#include <winuser32.mqh>
#include <bclib.mq4>

#define BSS 0
#define SBB 1

string PossibleRings[3,2]={
       "EURUSD  EURGBP  GBPUSD","*//",
       "EURJPY  EURGBP  GBPJPY","/**",
       "EURJPY  EURUSD  USDJPY","/**"};              

string Currencies[8]={"EUR","USD","GBP","JPY","CHF","AUD","CAD","NZD"};       
       
int RingsCount=0;
string RealRings[10,4];
int RealMagics[10];
bool SomethingNotOk=False;


//***********************************************************************
//                    Currency coding functions
//***********************************************************************

int CrrcyToInt(string crrcy)
    {
    string uc;
    int i;
    uc=Upper(crrcy);
    for (i=0;i<8;i++)
       {
       if (Currencies[i]==crrcy||Currencies[i]==uc)
         {
         return(i);       
         }    
       }
    return(-1);
    }   
    
string IntToCrrcy(int in)
    {
    return(Currencies[in]);
    }
        
    
int ContractToInt(string contract)    
    {
    int res,c1,c2;
    string cwork,aphere;
    aphere="";
    if (StringLen(contract)>6)    
       {
       aphere=SubstrBetween(contract,6,StringLen(contract));
       cwork=SubstrBetween(contract,1,6);
       }
    else    
       cwork=contract;
    c1=CrrcyToInt(SubstrBetween(cwork,0,2));
    c2=CrrcyToInt(SubstrBetween(cwork,3,5));
    res=c1*8+c2;
    return(res);
    }
    
string IntToContract(int in,string ap)
    {
    int c1,c2;
    string res;
    c2=Round(MathMod(in,8));
    c1=Round(MathDiv(in,8));
    res=Currencies[c1]+Currencies[c2]+ap;
    return(res);
    }

int PositionToMagic(string c1,string c2, string c3)
    {
    int res;
    res=ContractToInt(c1);
    res=res*64;
    res=res+ContractToInt(c2);
    res=res*64;
    res=res+ContractToInt(c3);
    return(res);
    }
    
void MagicToPosition(int magic,string &c1,string &c2,string &c3)
    {
    int r1,r2,r3;
    r3=MathRound(MathMod(magic,64));
    magic=MathDiv(magic,64);
    r2=MathRound(MathMod(magic,64));
    magic=MathDiv(magic,64);
    r1=magic;
    c1=IntToContract(r1,Appendix);
    c2=IntToContract(r2,Appendix);
    c3=IntToContract(r3,Appendix);       
    }
    
int GetRingPosition(int magic)
    {
    int res=-1;
    int i;
    for (i=0;i<RingsCount;i++)
       {
       if (RealMagics[i]==magic)
          {
          res=i;
          break;
          }
       }
    return(res);
    }

//***********************************************************************

string Crr(string contract,int whichone)
  {
  string res;
  if (whichone==1)
     {
     res=SubstrBetween(contract,0,2);  
     }
  else
     {
     res=SubstrBetween(contract,3,5);
     }
  return(res);
  }
  

string GotOrPaid(int optype,double volume,string contract,int whichone)
  {
  string res;
  if (optype==OP_BUY&&whichone==1)
     {
     res=StringConcatenate("we got ",DoubleToStr(volume,0)," ",Crr(contract,whichone));
     }
  if (optype==OP_BUY&&whichone==2)
     {
     res=StringConcatenate("we paid ",DoubleToStr(volume,0)," ",Crr(contract,whichone));
     }
  if (optype==OP_SELL&&whichone==2)
     {
     res=StringConcatenate("we got ",DoubleToStr(volume,0)," ",Crr(contract,whichone));
     }
  if (optype==OP_SELL&&whichone==1)
     {
     res=StringConcatenate("we paid ",DoubleToStr(volume,0)," ",Crr(contract,whichone));
     }
  return(res);     
  }

int RevRingType(int RingType)
    {
    int res;
    if (RingType==BSS)
       res=SBB;
    if (RingType==SBB)
       res=BSS;
    return(res);
    }

double CalculateRingFPI(int RingPos,int RingType)
  {
  double p1,p2,p3,res;
  string s1,s2,s3;
  if (RingType==BSS)
     {
     p1=MarketInfo(RealRings[RingPos,0],MODE_ASK);
     p2=MarketInfo(RealRings[RingPos,1],MODE_BID);
     p3=MarketInfo(RealRings[RingPos,2],MODE_BID);
     }
  if (RingType==SBB)
     {
     p1=MarketInfo(RealRings[RingPos,0],MODE_ASK);
     p2=MarketInfo(RealRings[RingPos,1],MODE_BID);
     p3=MarketInfo(RealRings[RingPos,2],MODE_BID);
     }
  s1=StringElement(RealRings[RingPos,3],0);
  s2=StringElement(RealRings[RingPos,3],1);
  s3=StringElement(RealRings[RingPos,3],2);
  res=1;
  if (s1=="*")
     res=res*p1;
  else
     res=res/p1;
  if (s2=="*")
     res=res*p2;
  else
     res=res/p2;
  if (s3=="*")
     res=res*p3;
  else
     res=res/p3;
  return(res);
  }  
  

void GetActiveRing(int &ActiveRing, int &RingType)
  {
  ActiveRing=-1;
  RingType=-1;
  int magic;
  int ntrades=OrdersTotal(); 
  if (ntrades!=0)
    {
    OrderSelect(0,SELECT_BY_POS,MODE_TRADES);
    magic=OrderMagicNumber();
    RingType=MathRound(MathMod(magic,2));
    ActiveRing=MathDiv(magic,2);
    }
  }

void DoCalculus(int Position, int RingType)
  {
  double available,r1,r2,r3;
  double first_volume,second_volume,second_volume_to_minus,second_volume_to_plus,dminus,dplus,required;
  double pricecl;  
  double first,second;
  string c1,c2,c3,etype,etext,eexpl;
  int op1,op2,op3,magic,ticket1,ticket2,ticket3,Gle;
  c1=RealRings[Position,0];
  c2=RealRings[Position,1];
  c3=RealRings[Position,2];
  magic=RealMagics[Position]*2+RingType;
  available=AccountLeverage()*MathRound(AccountEquity()*MarginUsage/100);
  if (RingType==BSS)
     {
     r1=MarketInfo(c1,MODE_ASK);
     op1=OP_BUY;
     r2=MarketInfo(c2,MODE_BID);
     op2=OP_SELL;
     r3=MarketInfo(c3,MODE_BID);     
     op3=OP_SELL;
     }
  else
     {
     r1=MarketInfo(c1,MODE_BID);
     op1=OP_SELL;
     r2=MarketInfo(c2,MODE_ASK);
     op2=OP_BUY;
     r3=MarketInfo(c3,MODE_ASK);     
     op3=OP_BUY;
     }  
  // example
  // [DIRECTION] EURJPY BIG_VOLUME * R1 = V1
  // [REV.DIR.]  EURUSD BIG_VOLUME * R2 = V2
  // [REV.DIR.]  USDJPY V1 * R3 = V3     
  first_volume=MathRound(available/(2+r2));    //maximum for big_volume
  first_volume=1000*Truncate(first_volume/1000); //cutting garbage
  required=first_volume*r2;
  second_volume_to_minus=Truncate(required/1000)*1000;
  second_volume_to_plus=(Truncate(required/1000)+1)*1000;
  dminus=required-second_volume_to_minus;
  dplus=second_volume_to_plus-required;
  if (dminus<=dplus)
     second_volume=MathRound(second_volume_to_minus/1000)*0.01;  //converting to lots
  else
     second_volume=MathRound(second_volume_to_plus/1000)*0.01;   //converting to lots
  first_volume=MathRound(first_volume/1000)*0.01;                  //converting to lots  
  
  first=first_volume/0.01*1000;
  second=second_volume/0.01*1000;
  
  ClearSplash();
  SplashLn(StringConcatenate(OrderType2Str(op2)," ",c2," :: ",GotOrPaid(op2,first,c2,1)," ",GotOrPaid(op2,first*r2,c2,2)));
  SplashLn(StringConcatenate(OrderType2Str(op1)," ",c1," :: ",GotOrPaid(op1,first,c1,1)," ",GotOrPaid(op1,first*r1,c1,2)));
  SplashLn(StringConcatenate(OrderType2Str(op3)," ",c3," :: ",GotOrPaid(op3,second,c3,1)," ",GotOrPaid(op2,second*r3,c3,2)));
  SplashLn("");
  SplashLn(StringConcatenate("meaning ",DoubleToStr(first_volume,2)," on ",c1," and ",c2," and ",DoubleToStr(second_volume,2)," on ",c3));
  SplashLn("");
  SplashLn("Exposure");
  SplashLn(StringConcatenate(DoubleToStr(MathAbs(first*r2-second),0)," on ",Crr(c3,1)));
  SplashLn(StringConcatenate(DoubleToStr(MathAbs(first*r1-second*r3),0)," on ",Crr(c3,2)));
  SplashLn("");
  SplashLn("Normal exposure under <Impeccable Hedge> assumption");
  SplashLn(StringConcatenate(DoubleToStr(MathAbs(first*r1-first*r2*r3),0)," on ",Crr(c3,2)));
 SplashScreen();  
     
 }
  
void Position_Destroy(int ActiveRing)
  {  
  int magic,i;
  int RingType;    
  double pricecl;
  string os;
  int ntrades=OrdersTotal(); 
  if (ntrades!=0)
    {
    for (i=0;i<ntrades;i++)
       {
       OrderSelect(0,SELECT_BY_POS,MODE_TRADES);
       magic=OrderMagicNumber();
       RingType=MathRound(MathMod(magic,2));
       if (MathDiv(magic,2)==ActiveRing)
         {
         os=OrderSymbol();
         if (OrderType()==OP_BUY)
            pricecl=MarketInfo(os,MODE_BID);
         if (OrderType()==OP_SELL)
            pricecl=MarketInfo(os,MODE_ASK);                     
         OrderClose(OrderTicket(),OrderLots(),pricecl,Slippage,Green);                
         SetTextColor(Green);
         WriteLn("Removed trade on "+os);
         SetTextColor(White);         
         }
       }       
    }
  }


//+------------------------------------------------------------------+
//| expert MAIN function                                             |
//+------------------------------------------------------------------+


  
    

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
  int rels=Round(ArraySize(PossibleRings)/2);
  int i;
  int rcount;
  string c1,c2,c3,op;
  bool cex1,cex2,cex3,opok;
  rcount=0;
  ClearText();
  for (i=0;i<rels;i++)
     {
     opok=false;
     c1=ReturnNthWord(PossibleRings[i,0],1);
     c2=ReturnNthWord(PossibleRings[i,0],2);
     c3=ReturnNthWord(PossibleRings[i,0],3);
     op=PossibleRings[i,1];
     cex1=ContractExists(StringConcatenate(c1,Appendix));
     cex2=ContractExists(StringConcatenate(c2,Appendix));
     cex3=ContractExists(StringConcatenate(c3,Appendix));
 
     if (cex1==True&&cex2==True&&cex3==True)
        {
        RealRings[rcount,0]=StringConcatenate(c1,Appendix);
        RealRings[rcount,1]=StringConcatenate(c2,Appendix);
        RealRings[rcount,2]=StringConcatenate(c3,Appendix);
        RealRings[rcount,3]=op;        
        RealMagics[rcount]=PositionToMagic(c1,c2,c3);
        rcount=rcount+1;
        }
     else
        {
        ClearSplash();        
        if (cex1==false)
           SplashLn(StringConcatenate("Contract not found :",StringConcatenate(c1,Appendix))); 
        if (cex2==false)                      
           SplashLn(StringConcatenate("Contract not found :",StringConcatenate(c2,Appendix)));
        if (cex3==false)                      
           SplashLn(StringConcatenate("Contract not found :",StringConcatenate(c3,Appendix))); 
        SplashScreen();
        return(1);         
        }   
     } 
  RingsCount=rcount;
  
  //*****************************************************************************
              
  return(0);
  }
  

  
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
  int RingFromList,RingType;
  RingFromList=2;
  RingType=BSS;
  DoCalculus(RingFromList,RingType);
  return(0);
  }  



